outputDir = file(params.outputDir, type: 'dir')

process FASTP {
  tag "fastp"
  container 'quay.io/biocontainers/fastp:0.20.0--hdbcaa40_0'

  publishDir "${outputDir}"

  input:
    tuple val(srr_id), path(fastq1), path(fastq2)
    val threads
    val cut_right_window_size
    val cut_right_mean_quality
    val trim_tail1
    val length_required

  output:
    tuple path("${fastq1.simpleName}.trimmed.fastq"),
          path("${fastq2.simpleName}.trimmed.fastq"),
          path("${fastq1.simpleName}.unpaired.trimmed.fastq"),
          path("${fastq2.simpleName}.unpaired.trimmed.fastq"),
          emit: reads
    
    tuple val(srr_id),
          path("${fastq1.simpleName}.trimmed.fastq"),
          path("${fastq2.simpleName}.trimmed.fastq"), emit: paired_fastq
    path "fastp/*.html", emit: html_report
    path "fastp/*.json", emit: json_report

  script:
  def fastpthreadsArgument = params.threads_fastp ? "--thread ${threads}" : ''
  def fastpcut_rigthArgument = params.cut_right ? "--cut_right" : ''
  def fastpcut_rigth_window_sizeArgument = params.cut_right_window_size ? "--cut_right_window_size ${cut_right_window_size}" : ''
  def fastpcut_rigth_mean_qualityArgument = params.cut_right_mean_quality ? "--cut_right_mean_quality ${cut_right_mean_quality}" : ''
  def fastptrim_tail1Argument = params.trim_tail1 ? "--trim_tail1 ${trim_tail1}" : ''
  def fastplength_requiredArgument = params.length_required ? "--length_required ${length_required}" : ''
  """
  mkdir fastp

  SAMPLE="\$(echo ${fastq1.simpleName} | cut -f 1 -d '_')"

  fastp \\
    -o ${fastq1.simpleName}.trimmed.fastq \\
    -O ${fastq2.simpleName}.trimmed.fastq \\
    --unpaired1 ${fastq1.simpleName}.unpaired.trimmed.fastq \\
    --unpaired2 ${fastq2.simpleName}.unpaired.trimmed.fastq \\
    ${fastpcut_rigthArgument} \\
    ${fastpcut_rigth_mean_qualityArgument} \\
    ${fastpcut_rigth_window_sizeArgument} \\
    -i $fastq1 \\
    -I $fastq2 \\
    ${fastplength_requiredArgument} \\
    ${fastptrim_tail1Argument} \\
    --json fastp/\${SAMPLE}.fastp.json \\
    --html fastp/\${SAMPLE}.fastp.html \\
    ${fastpthreadsArgument}
  """
}
